/*
  Copyright (c) 2008, 2011 Gordon Gremme <gordon@gremme.org>

  Permission to use, copy, modify, and distribute this software for any
  purpose with or without fee is hereby granted, provided that the above
  copyright notice and this permission notice appear in all copies.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef BED_PARSER_H
#define BED_PARSER_H

#include "core/queue_api.h"

#define BED_FEATURE_TYPE        "BED_feature"
#define BED_THICK_FEATURE_TYPE  "BED_thick_feature"
#define BED_BLOCK_TYPE          "BED_block"

typedef struct GtBEDParser GtBEDParser;

GtBEDParser* gt_bed_parser_new(void);
void         gt_bed_parser_delete(GtBEDParser*);
int          gt_bed_parser_parse(GtBEDParser*, GtQueue *genome_nodes,
                                 const char *filename, GtError*);
void         gt_bed_parser_set_feature_type(GtBEDParser*, const char*);
void         gt_bed_parser_set_thick_feature_type(GtBEDParser*, const char*);
void         gt_bed_parser_set_block_type(GtBEDParser*, const char*);

#endif
